import { 
  VApp,
  VBtn,
  VCard,
  VCardActions, 
  VCardText,
  VCardTitle,
  VCheckbox,
  VContent,
  VDialog,
  VFlex,
  VLayout,
  VMenu,
  VProgressLinear,
  VSnackbar,
} from 'vuetify/lib'

import Vuetify from 'vuetify'
import Vue from 'vue'

Vue.use(Vuetify)

export {
  VApp,
  VBtn,
  VCard,
  VCardActions, 
  VCardText,
  VCardTitle,
  VCheckbox,
  VContent,
  VDialog,
  VFlex,
  VLayout,
  VMenu,
  VProgressLinear,
  VSnackbar,
}